#include "OgreStableHeaders.h"
#include "OgreSolidHalfBoundingBox.h"

#include "OgreSimpleRenderable.h"
#include "OgreHardwareBufferManager.h"
#include "OgreCamera.h"
#include "OgreMaterialManager.h"
//#include <windows.h>

namespace Ogre {

#define POSITION_BINDING 0
//-----------------------------------------------------------------------
SolidHalfBoundingBox::SolidHalfBoundingBox()
{
	SetOcclusionQueryMaterial();
	
	mRenderOp.vertexData->vertexCount = 8;
    mRenderOp.operationType = RenderOperation::OT_TRIANGLE_FAN; 
}
//-----------------------------------------------------------------------
void SolidHalfBoundingBox::SetupBoundingBoxVertices(const AxisAlignedBox& aab, 
													const bool isFirstHalf) 
{
	Vector3 vmax = aab.getMaximum();
	Vector3 vmin = aab.getMinimum();
		
	//Real sqLen = std::max(vmax.squaredLength(), vmin.squaredLength());
    //mRadius = Math::Sqrt(sqLen);
	
	Real maxx = vmax.x;
	Real maxy = vmax.y;
	Real maxz = vmax.z;
	
	Real minx = vmin.x;
	Real miny = vmin.y;
	Real minz = vmin.z;
	
	// fill the vertex buffer: 12 lines with 2 endpoints each make up a box
    HardwareVertexBufferSharedPtr vbuf =
        mRenderOp.vertexData->vertexBufferBinding->getBuffer(POSITION_BINDING);     

    float* pPos = static_cast<float*>(
        vbuf->lock(HardwareBuffer::HBL_DISCARD));

	//     5+------+7
	//     /|     /|
	//    / |    / |
	//   / 1+---/--+3
	// 2+------+6 /    y   z
	//  | /    | /     |  /
	//  |/     |/      |/
	// 0+------+4      *---x

	// fan 1
	if(isFirstHalf)
	{
		*pPos++ = minx; *pPos++ = maxy; *pPos++ = maxz; //011
		*pPos++ = minx; *pPos++ = miny; *pPos++ = maxz; //001
		*pPos++ = maxx; *pPos++ = miny; *pPos++ = maxz; //101
		*pPos++ = maxx; *pPos++ = maxy; *pPos++ = maxz; //111
		
		*pPos++ = maxx; *pPos++ = maxy; *pPos++ = minz; //110
		*pPos++ = minx; *pPos++ = maxy; *pPos++ = minz; //010
		*pPos++ = minx; *pPos++ = miny; *pPos++ = minz; //000
		*pPos++ = minx; *pPos++ = miny; *pPos++ = maxz; //001
	}
	else
	// fan 2
	{
		*pPos++ = maxx; *pPos++ = miny; *pPos++ = minz; //100
		*pPos++ = minx; *pPos++ = miny; *pPos++ = minz; //000 					
		*pPos++ = minx; *pPos++ = maxy; *pPos++ = minz; //010
		*pPos++ = maxx; *pPos++ = maxy; *pPos++ = minz; //110

		*pPos++ = maxx; *pPos++ = maxy; *pPos++ = maxz; //111
		*pPos++ = maxx; *pPos++ = miny; *pPos++ = maxz; //101
		*pPos++ = minx; *pPos++ = miny; *pPos++ = maxz; //001
		*pPos++ = minx; *pPos++ = miny; *pPos++ = minz; //000
	}

    vbuf->unlock();
}
//-----------------------------------------------------------------------
void SolidHalfBoundingBox::SetupBoundingBox(const AxisAlignedBox& aabb, 
											const bool isFirstHalf) 
{
	// init the vertices to the aabb
	SetupBoundingBoxVertices(aabb, isFirstHalf);

    // setup the bounding box of this SimpleRenderable
	//setBoundingBox(aabb);
}
//-----------------------------------------------------------------------
void SolidHalfBoundingBox::SetOcclusionQueryMaterial( void )
{
	m_pMaterial = MaterialManager::getSingleton().getByName("OcclusionQuery");

	if(m_pMaterial.isNull())
	{
		m_pMaterial = MaterialManager::getSingleton().
			create("OcclusionQuery", ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
	}

	m_pMaterial->setColourWriteEnabled(false);
	m_pMaterial->setDepthWriteEnabled(false);
	m_pMaterial->setLightingEnabled(false);

	setMaterial("OcclusionQuery");
}

} // namespace Ogre

